@@ -1,7 +1,7 @@
-from copy import deepcopy
 import json
 import logging
 import os
+from copy import deepcopy
 
 import slugid
 
@@ -21,6 +21,7 @@ _track_default_position = {
     "bedlike": "top",
     "horizontal-bar": "top",
     "horizontal-chromosome-labels": "top",
+    "chromosome-labels": "top",
     "horizontal-gene-annotations": "top",
     "horizontal-heatmap": "top",
     "horizontal-1d-heatmap": "top",
@@ -73,6 +74,8 @@ class Track(Component):
     ----------
     track_type : str
         The type of track (e.g. 'heatmap', 'line')
+    position: str
+        The position of the track (e.g. 'top')
     tileset : :class:`Tileset`
         A Tileset being displayed in this track
     file_url: str
@@ -105,6 +108,9 @@ class Track(Component):
                 track_type = kwargs.pop("type")
             else:
                 raise ValueError("Track type is required.")
+        else:
+            if "type" in kwargs:
+                kwargs.pop("type")
 
         if not position:
             position = _track_default_position[track_type]
@@ -149,8 +155,18 @@ class Track(Component):
         Change an attribute of this track and return a new copy.
         """
         conf = self.conf.copy()
+        position = self.position
+        track_type = self.conf["type"]
+
+        if "track_type" in kwargs:
+            track_type = kwargs["track_type"]
+        if "position" in kwargs:
+            position = kwargs["position"]
+
         conf.update(kwargs)
-        return self.__class__(conf["type"], position=self.position, tileset=self.tileset, **conf)
+        return self.__class__(
+            track_type=track_type, position=position, tileset=self.tileset, **conf
+        )
 
     def change_options(self, **kwargs):
         """
@@ -214,8 +230,12 @@ class DividedTrack(Track):
                 f"Different track types: {numerator.conf['type']}, {denominator.conf['type']}"
             )
 
-        if json.dumps(numerator.conf["options"]) != json.dumps(denominator.conf["options"]):
-            logger.warn("Tracks have different options, so we're using the first track's")
+        if json.dumps(numerator.conf["options"]) != json.dumps(
+            denominator.conf["options"]
+        ):
+            logger.warn(
+                "Tracks have different options, so we're using the first track's"
+            )
 
         numerator_server = numerator.conf["server"]
         numerator_uuid = numerator.conf["tilesetUid"]
@@ -251,9 +271,17 @@ class DividedTrack(Track):
         Change an attribute of this track and return a new copy.
         """
         conf = self.conf.copy()
+        position = self.position
+        track_type = self.conf["type"]
+
+        if "track_type" in kwargs:
+            track_type = kwargs["track_type"]
+        if "position" in kwargs:
+            position = kwargs["position"]
+
         conf.update(kwargs)
 
-        return Track(conf["type"]).from_dict(conf)
+        return Track(track_type=track_type, position=position).from_dict(conf)
 
 
 class CombinedTrack(Track):
@@ -463,7 +491,9 @@ class View(Component):
                 # position has to be passed in as part of the parameter
                 # array so that the constructor can be called with it as
                 # a parameter
-                self.add_track(track=klass.from_dict({"position": position, **track_conf}))
+                self.add_track(
+                    track=klass.from_dict({"position": position, **track_conf})
+                )
 
         return self
 
@@ -526,7 +556,27 @@ class View(Component):
 class ViewConf(Component):
     """Configure a dashboard"""
 
-    def __init__(self, views=[], location_syncs=[], value_scale_syncs=[], zoom_syncs=[]):
+    def __init__(
+        self, views=[], location_syncs=[], value_scale_syncs=[], zoom_syncs=[]
+    ):
+        """A Python representation of a HiGlass viewconf.
+
+        Parameters
+        ----------
+        views: list[list]
+            A list of View objects which compose the viewable scene
+        location_syncs: list[list]
+            A list of lists of Views to be synced
+        value_scale_syncs: list[list]
+            A list containing the value scale syncs. Each sync can be
+            one of:
+                1. a list of (View, Track) tuples
+                2. a list of Tracks (assumes that there is only one view)
+                3. a list of strings of the form "{viewUid}.{trackUid}"
+        location_syncs: list[list]
+            A list of lists of Views to be synced
+
+        """
 
         self.conf = {
             "editable": True,
@@ -677,7 +727,9 @@ class ViewConf(Component):
         else:
             raise ValueError("Not a valid viewconf server")
 
-        endpoint = urlunsplit((parts.scheme, parts.netloc, "api/v1/viewconfs", "d=" + conf_id, ""))
+        endpoint = urlunsplit(
+            (parts.scheme, parts.netloc, "api/v1/viewconfs", "d=" + conf_id, "")
+        )
 
         conf = requests.get(endpoint).json()
 
